# Generated by Django 4.0.2 on 2022-08-28 14:00

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MenuItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('hook_function', models.CharField(max_length=500)),
                ('icon_classes', models.CharField(max_length=150)),
                ('text', models.CharField(max_length=150)),
                ('url', models.CharField(blank=True, default=None, max_length=2048, null=True)),
                ('rank', models.IntegerField(default=1000)),
                ('hide', models.BooleanField(default=False)),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='menu.menuitem')),
            ],
        ),
    ]
