{% load static %}
<!-- allianceauth.theme.{{ theme.name }} JS -->
{% if theme.js_template %}
    {% include theme.css_template %}
{% else %}
    {% for x in theme.js %}
        <script src="{{ x.url }}" integrity="{{ x.integrity }}" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
    {% endfor %}
{% endif %}
<!-- allianceauth.theme.{{ theme.name }} JS Ends-->
