# Generated by Django 1.10.5 on 2017-03-22 23:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0007_unique_id_name'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='eveapikeypair',
            name='user',
        ),
        migrations.RemoveField(
            model_name='eveallianceinfo',
            name='is_blue',
        ),
        migrations.RemoveField(
            model_name='evecharacter',
            name='api_id',
        ),
        migrations.RemoveField(
            model_name='evecharacter',
            name='user',
        ),
        migrations.RemoveField(
            model_name='evecorporationinfo',
            name='is_blue',
        ),
        migrations.DeleteModel(
            name='EveApiKeyPair',
        ),
    ]
