# Generated by Django 4.0.10 on 2023-05-28 15:36

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("authentication", "0020_userprofile_language_userprofile_night_mode"),
    ]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="language",
            field=models.CharField(
                blank=True,
                choices=[
                    ("en", "English"),
                    ("de", "German"),
                    ("es", "Spanish"),
                    ("zh-hans", "Chinese Simplified"),
                    ("ru", "Russian"),
                    ("ko", "Korean"),
                    ("fr", "French"),
                    ("ja", "Japanese"),
                    ("it", "Italian"),
                    ("uk", "Ukrainian"),
                ],
                default="",
                max_length=10,
                verbose_name="Language",
            ),
        ),
    ]
