# Generated by Django 4.0.8 on 2023-02-05 07:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('menu', '0002_alter_menuitem_hook_function_and_more'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='menuitem',
            index=models.Index(fields=['rank'], name='menu_menuit_rank_e880ab_idx'),
        ),
    ]
