<!-- Start Datatables-js from cdnjs -->
<script src="https://cdnjs.cloudflare.com/ajax/libs/datatables.net/1.13.4/jquery.dataTables.min.js" integrity="sha512-8mDDicdS5IeRnHI+un7skH0RJoB2QMMJmORCHizlxul5jgf9tSWJwpE1eSl/2a9xFQYGOwXLr0Lh5SJPnL4IOw==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/datatables.net-bs5/1.13.4/dataTables.bootstrap5.min.js" integrity="sha512-KFdmxVdAssPxrj4mZh1c01AbGXMAmXmBsO4Tc/GG5+kNLqitTfUBpDHicyDwF7CaFV+pN1r808IOK+vHzWB8gw==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
<!-- End Datatables-js from cdnjs -->
