# Generated by Django 2.2.8 on 2020-01-06 11:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
        ('groupmanagement', '0011_requestlog_date'),
    ]

    operations = [
        migrations.AddField(
            model_name='authgroup',
            name='group_leader_groups',
            field=models.ManyToManyField(blank=True, help_text='Group leaders can process group requests for this group specifically. Use the auth.group_management permission to allow a user to manage all groups.', related_name='leads_group_groups', to='auth.Group'),
        )
    ]
