<form action="{% url 'theme' %}?next={{ next|urlencode }}" method="post">
    {% csrf_token %}
    <select name="theme" value="theme" class="form-select" aria-label="" onchange="this.form.submit()">
        <option selected>Select Theme</option>
        {% for theme in themes %}
            <option value="{{ theme.get_name }}">{{ theme.name }}</option>
        {% endfor %}
    </select>
</form>
