# Generated by Django 1.10.2 on 2016-12-12 01:11

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AuthTS',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('auth_group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Group')),
            ],
            options={
                'db_table': 'teamspeak3_authts',
                'verbose_name': 'Auth / TS Group',
            },
        ),
        migrations.CreateModel(
            name='TSgroup',
            fields=[
                ('ts_group_id', models.IntegerField(primary_key=True, serialize=False)),
                ('ts_group_name', models.CharField(max_length=30)),
            ],
            options={
                'db_table': 'teamspeak3_tsgroup',
                'verbose_name': 'TS Group',
            },
        ),
        migrations.CreateModel(
            name='UserTSgroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ts_group', models.ManyToManyField(to='teamspeak3.TSgroup')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'teamspeak3_usertsgroup',
                'verbose_name': 'User TS Group',
            },
        ),
        migrations.AddField(
            model_name='authts',
            name='ts_group',
            field=models.ManyToManyField(to='teamspeak3.TSgroup'),
        ),
    ]
