# Generated by Django 3.2.8 on 2021-10-26 09:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0018_alter_state_name_length'),
        ('authentication', '0018_state_member_factions'),
    ]

    operations = [
    ]
