# Generated by Django 2.2.9 on 2020-01-30 13:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('groupmanagement', '0012_group_leads'),
    ]

    operations = [
        migrations.AlterField(
            model_name='requestlog',
            name='date',
            field=models.DateTimeField(auto_now_add=True),
        ),
    ]
