# Generated by Django 4.0.7 on 2022-08-14 16:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0016_character_names_are_not_unique'),
    ]

    operations = [
        migrations.AlterField(
            model_name='eveallianceinfo',
            name='alliance_name',
            field=models.CharField(max_length=254, db_index=True),
        ),
        migrations.AlterField(
            model_name='evecorporationinfo',
            name='corporation_name',
            field=models.CharField(max_length=254, db_index=True),
        ),
    ]
