# Generated by Django 3.2.8 on 2021-10-20 05:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0017_remove_fleetup_permission'),
    ]

    operations = [
        migrations.AlterField(
            model_name='state',
            name='name',
            field=models.CharField(max_length=32, unique=True),
        ),
    ]
